
import typer

from cli import django
from cli import schedule
from cli import webapp
from cli import path

help = """This is a new experimental PythonAnywhere cli client.

It was build with typer & click under the hood.
"""

app = typer.Typer(help=help)
app.add_typer(django.app, name="django", help="Makes Django Girls tutorial projects deployment easy")
app.add_typer(schedule.app, name="schedule", help="Manage scheduled tasks")
app.add_typer(webapp.app, name="webapp", help="Everything for web apps")
app.add_typer(path.app, name="path", help="Perform some operations on files")


if __name__ == "__main__":
    app(prog_name="pa")
